class LinearMap(object):
	def __init__(self):
		self.items = []

	def add(self, k, v):
		self.items.append((k, v))

	def get(self, k):  
		for key, val in self.items:  
			if key == k:  
				return val  
		raise KeyError  
  
  
class BetterMap(object):
	def __init__(self, n=100):  
		self.maps = []
		for i in range(n):  
			self.maps.append(LinearMap())  

	def find_map(self, k):  
		index = hash(k) % len(self.maps)  
		return self.maps[index]

  	def add(self, k, v):  
		m = self.find_map(k)  
		m.add(k, v)

	def get(self, k):  
		m = self.find_map(k)  
		return m.get(k)
  
class HashMap(object):  
	def __init__(self):  
		self.maps = BetterMap(2)  
		self.num = 0  

	def get(self, k):  
		return self.maps.get(k)  

	def add(self, k, v):  
		if self.num == len(self.maps.maps):  
			self.resize()  

		self.maps.add(k, v)  
		self.num += 1  

	def resize(self):  
		new_map = BetterMap(self.num * 2)  

		for m in self.maps.maps:  
			for k, v in m.items:  
				new_map.add(k, v)  

		self.maps = new_map
	
	def length(self):
		return self.num
	pass


def main(script): 
    import string   
    m = HashMap()  
    s = string.ascii_lowercase  
  
    for k, v in enumerate(s):  
        m.add(k, v)  
  
    for k in range(len(s)):  
        print k, m.get(k)  


if __name__ == '__main__':   
    import sys 
    main(*sys.argv)  


#!/usr/bin/env python             
# coding: utf-8
import tinify
import os
import time
import shutil
import  util
from hashmap import HashMap
import lcba.globalVar as glo

# 常量: 服务器最大容错次数
MAX_SERVER_ERROR_COUNT = 3
# 常量: 图片最大容错次数
MAX_PIC_ERROR_COUNT = 3
# 常量:账号列表
ACCOUNT_KEY_LIST = [
    "WaNaNSYc84TNr8DZpgZFRo8QnxhSaV3L",  # use hongc@jumei.com register
    "tdSdRzIvicVO3oTfrXZ6374P7uyBx-Kf",  # use changqiangl@jumei.com register
    "Jltmnr4b7xkQedUyiza-v12FZpIOPG8O",  # use test1_jumei@mail.com register
    "tFrVjiAQyYshUKvGDGuCZZa0cmW_VuM9",  # use xiangc register
]

# 全局变量:服务器错误次数
serverErrorCount = 0
# 全局变量:单次图片错误次数
picErrorCount = 0
# 全局变量:当前帐号索引
curApiKeyIndex = 0

JUST_CHECK = None


# mac 安装python -m pip install --upgrade pip
# pip install --upgrade tinify
# 怎样使用请访问 https://tinypng.com/developers/reference/python
################################################
# 对图片进行实际压缩,请求tinyPng压缩图片 接受xxx.png和xxx.jpeg的图片
###############################################
def compressSinglePic(src, dest):
    try:
        # if方案一使用文件作为源
        source = tinify.from_file(src)
        source.to_file(dest)

    # elseif方案二使用文件buffer来作为源
    # with open("unoptimized.jpg", 'rb') as source:
    # source_data = source.read()
    # result_data = tinify.from_buffer(source_data).to_buffer()

    # elseif方案三使用url来作为源
    # source = tinify.from_url("https://cdn.tinypng.com/images/panda-happy.png")
    # source.to_file("optimized.jpg")
    # pass
    except tinify.AccountError, e:
        # 账号错误，请重试或者换账号
        # Verify your API key and account limit.
        print "The error message is: %s  index:%s" % (e.message, curApiKeyIndex)
        return -1
    except tinify.ClientError, e:
        # 不建议重试请求，可能是图片有问题
        # Check your source image and request options.
        print "The error picture is: %s" % src
        print "The error message is: %s" % e.message
        return -2
    except tinify.ServerError, e:
        # 建议重试请求；官方说法一般是几分钟之后会恢复
        # Temporary issue with the Tinify API.
        print "The error picture is: %s  server error count is:" % (src, picErrorCount)
        print "The error message is: %s" % e.message
        return -3
    except tinify.ConnectionError, e:
        # 检查网络连接或者重试
        # A network connection error occurred.
        print "The error picture is: %s  pic error count is:" % (src, picErrorCount)
        print "The error message is: %s" % e.message
        return -4
    except Exception, e:
        # 未知异常
        # Something else went wrong, unrelated to the Tinify API.
        print "The error picture is: %s " % src
        print "The error message is: %s" % e.message
        return -5
    return 0


pass


################################################
# 对图片进行逻辑压缩处理
###############################################
def compressSinglePicLogic(src, dest, keyName, mintor_md5_filename):
    global picErrorCount
    global serverErrorCount
    global curApiKeyIndex

    # print "compressSinglePicLogic()...%s" %src
    ret = compressSinglePic(src, dest)
    if 0 == ret:
        oldSize = os.path.getsize(src)
        newSize = os.path.getsize(dest)
        print "%s compressed from:%s to:%s -------->saving:%s" % (keyName, oldSize, newSize, oldSize - newSize)
        # 3，删除源文件，把新文件重新命名为源文件
        shutil.move(dest, src)
        # 1，在md5文件中删除老的一行，并把新的md5写入
        util.deleteLine(mintor_md5_filename, keyName)
        newMd5 = util.generateMd5_2(src)
        md5Line = "%s--MD5-->%s" % (keyName, newMd5)
        # print md5Line
        util.writeLine(mintor_md5_filename, md5Line)
    elif -1 == ret:
        curApiKeyIndex += 1
        validateApiKey(curApiKeyIndex)
        print "retry compressPic()..-1.src:%s" % src
        compressSinglePicLogic(src, dest, keyName, mintor_md5_filename)
    elif -2 == ret:
        util.safeQuit(None, None)
    elif -3 == ret:
        if serverErrorCount >= MAX_SERVER_ERROR_COUNT:
            util.safeQuit(None, None)
        else:
            serverErrorCount += 1
            print "we will retry in 2 minute later!!"
            time.sleep(60 * 2)
            print "retry compressPic()..-3.src:%s" % src
            compressSinglePicLogic(src, dest, keyName, mintor_md5_filename)
        pass
    elif -4 == ret:
        if picErrorCount >= MAX_PIC_ERROR_COUNT:
            util.safeQuit(None)
        else:
            picErrorCount += 1
            print "retry compressPic()..-4.src:%s" % src
            compressSinglePicLogic(src, dest, keyName, mintor_md5_filename)
        pass
    elif -5 == ret:
        util.safeQuit(None, None)
    else:
        util.safeQuit(None, None)
        pass
pass


################################################
# 验证账户
###############################################
def validateApiKey(index):
    global curApiKeyIndex
    if index >= len(ACCOUNT_KEY_LIST):
        util.safeQuit(None, None)
        return
    print "validateApiKey()... %s" % ACCOUNT_KEY_LIST[index]
    try:
        tinify.key = ACCOUNT_KEY_LIST[index]
        tinify.validate
    except tinify.Error, e:
        # Validation of API key failed.
        print "The error message is: %s  index:%s" % (e.message, curApiKeyIndex)
        if index >= ACCOUNT_KEY_LIST.size():
            util.safeQuit(None, None)
            return
        else:
            curApiKeyIndex += 1
            validateApiKey(curApiKeyIndex)
        pass
    pass


pass


################################################
# 检查受监视的图片目录
###############################################
def checkMintorDir(app_dir,mintor_dir_path, mintor_md5_filepath):
    print "checkMintorDir()...%s  from file: %s where module is:%s" % (mintor_dir_path, mintor_md5_filepath,app_dir)
    if not app_dir or not mintor_dir_path or not mintor_md5_filepath:
        print "###########ERROR###############"
        print "checkMintorDir param error app_dir:%s mintor_dir_path:%s mintor_md5_filepath:%s"%(app_dir,
                                                                                                mintor_dir_path,mintor_md5_filepath)
        return
    pass
    # 读区已经压缩过文件和md5到hashmap
    md5HashMap = HashMap()
    if os.path.exists(mintor_md5_filepath):
        for line in open(mintor_md5_filepath):
            # print line
            line = line.replace('\r', '').replace('\n', '').replace('\t', '')
            lineItems = line.rsplit('--MD5-->', 2)
            #print "%s   %s" %(lineItems[0],lineItems[1])
            md5HashMap.add(lineItems[0], lineItems[1])
        pass
    else:
        f = open(mintor_md5_filepath, 'w')
        f.close()
    pass

    # 遍历被监控的文件夹，计算新的md5值和老的md5Hashmap值是否相等
    for dirpath, dirnames, filenames in os.walk(mintor_dir_path):
        # print "dirpath:%s dirnames:%s filenames:%s" %(dirpath,dirnames,filenames)
        # if "build" not in dirpath and "intermediates" not in dirpath and "generated" not in dirpath and "outputs" not in dirpath and "gen" not in dirpath:
        #     continue
        for file in filenames:
            fullpath = os.path.join(dirpath, file)

            simplepath = fullpath.replace(app_dir, '')
            if simplepath and simplepath.startswith('/'):
                simplepath = simplepath[1:len(simplepath)]
            pass
            # print "this is: %s" %simplepath
            # 9.png 压缩了之后不能缩放
            if simplepath.endswith(".9.png"):
                print "ignore: %s" % simplepath
            elif simplepath.endswith(".png") or simplepath.endswith(".jpg"):
                oldMd5 = None
                try:
                    oldMd5 = md5HashMap.get(simplepath)
                except KeyError, e:
                    pass
                md5 = util.generateMd5_2(fullpath)
                print "\ncompressing %s oldMd5:%s md5:%s" % (simplepath, oldMd5, md5)
                if cmp(oldMd5, md5) == 0:
                    print "%s is already compressed!!! skiped" % simplepath
                else:
                    if JUST_CHECK:
                        title = "you have picture not compressed yet!!!"
                        content = "please local run:python lcba/lintCompress.py to compress Resources!!!"
                        print title
                        print content
                        glo.set_value('_reason',title)
                        glo.set_value('_title',title)
                        glo.set_value('_content',content)
                        util.safeQuit(None,None)
                    pass
                    # 2，上传tinypng压缩图片
                    compressSinglePicLogic(fullpath, fullpath + ".tiny", simplepath, mintor_md5_filepath)
                pass
            pass
        pass
    pass
pass

#监测出该目录有多少个图片文件夹
def getPictureDirs(app_dir):
    mintordirPaths = []
    for dirpath, dirnames, filenames in os.walk(app_dir):
        # print "dirpath:%s dirnames:%s filenames:%s" %(dirpath,dirnames,filenames)
        for dirname in dirnames:
            if "build" not in dirpath and "intermediates" not in dirpath and "generated" not in dirpath and "outputs" not in dirpath and "gen" not in dirpath:
                if dirname.startswith("asserts") or dirname.startswith("drawable") or dirname.startswith("mipmap"):
                    fullDirPath = os.path.join(dirpath, dirname)
                    print fullDirPath
                    mintordirPaths.append(fullDirPath)
                pass
            pass
        pass
    pass
    return mintordirPaths
pass

def compressModule(inDir=None,outDir=None,accountList=None,just_check=False):
    try:
        validateApiKey(curApiKeyIndex)
    except Exception, exc:
        print exc
        util.safeQuit(None,None)

    if not inDir or not outDir or not os.path.exists(inDir):
        print "compressModule()...param error inDir:%s outDir:%s" %(inDir,outDir)
        return

    if not os.path.exists(outDir):
        os.makedirs(r'%s' % outDir)
    pass

    global JUST_CHECK
    JUST_CHECK = just_check

    if accountList:
        global ACCOUNT_KEY_LIST
        ACCOUNT_KEY_LIST.extend(accountList)

    mintordirPaths = getPictureDirs(inDir)
    if mintordirPaths:
        for mintor_dir_name in mintordirPaths:
            tmp = mintor_dir_name.split('/');
            length = len(tmp)
            if length > 0:
                fix_name = tmp[length - 1]
                mintor_md5_filename = fix_name + "_md5.txt"
                # print mintor_md5_filename\
                checkMintorDir(inDir,mintor_dir_name, os.path.join(outDir, mintor_md5_filename))
        pass
    pass
pass

################################################
# 主函数
###############################################
def main():
    curPath = '/Users/tory/working/Codes/JUMEI/android/app/'
    outDir = '/Users/tory/working/Codes/JUMEI/android/lcba/app/'
    compressModule(inDir=curPath,outDir=outDir)
pass

if __name__ == '__main__':
    main()



#!/usr/bin/env python             
# coding: utf-8
import hashlib
import globalVar as glo
import os
import shutil
import smtplib
from email.mime.text import MIMEText


################################################
#删除某个文件特定的一行
###############################################
def deleteLine(srcFile, targetString):
	with open(srcFile, 'r') as f:
		#print "open:%s" % srcFile
		with open(srcFile+".new", 'w') as g:
			#print "open:new file" 
			for line in f.readlines():
				if targetString not in line:             
					g.write(line)
				else :
					print "delete line: %s" %targetString
			pass
		pass
	pass
	if os.path.exists(srcFile+".new"):
		shutil.move(srcFile+".new", srcFile)
	pass	
pass

################################################
#向某个文件写入特定的一行
###############################################
def writeLine(srcFile, targetString):
	f=open(srcFile,'a')
	f.write(targetString)
	f.write("\n")
	f.close()
pass

################################################
#md5 计算方案一直接读取计算，但是会消耗大量内存
###############################################
def generateMd5(filename): 
	if not os.path.isfile(filename):
		print ("%s is not a file!!!"%(filename))
		return
         
	fd = open(filename,"r")  
	fcont = fd.read()
	fd.close()
	fmd5 = hashlib.md5(fcont)
	md5 = fmd5.hexdigest()
	#print ("%s --MD5--> %s"%(filename,md5))
	return md5
pass

################################################
#md5 计算方案二直接读取计算最大占用8K内存
###############################################
def generateMd5_2(fname):  
    """ 计算文件的MD5值 """  
    def read_chunks(fh):  
        fh.seek(0)  
        chunk = fh.read(8096)  
        while chunk:  
            yield chunk  
            chunk = fh.read(8096)  
        else: #最后要将游标放回文件开头  
            fh.seek(0)  
    m = hashlib.md5()  
    if isinstance(fname, basestring) and os.path.exists(fname):  
        with open(fname, "rb") as fh:  
            for chunk in read_chunks(fh):  
                m.update(chunk)  
    #上传的文件缓存 或 已打开的文件流  
    elif fname.__class__.__name__ in ["StringIO", "StringO"] or isinstance(fname, file):  
        for chunk in read_chunks(fname):  
            m.update(chunk)  
    else:  
        return ""  
    return m.hexdigest()
pass

################################################
#检查gradle 任务
###############################################
def haveGradleTask(workPath,gradlewPath,taskName):
	if not gradlewPath:
		print "not find work path!!!"
		return False

	tmpFilePath = os.path.join(workPath,"tasks.new")
	os.chdir(r'%s'%workPath)
	print "haveGradleTask()?...%s ??? wait..."%taskName
	cmd = '%s tasks --all > %s'%(gradlewPath,tmpFilePath)
	#print cmd
	os.system(cmd)
	ret = False

	if not os.path.exists(tmpFilePath):
		print "not find work path!!!-->%s"%tmpFilePath
		return ret
	pass
	with open(tmpFilePath) as f:
		for line in f.readlines():
			if taskName in line:
				print "find gradlew task :%s"%taskName
				ret = True
				break
			pass
		pass
	pass

	if os.path.exists(tmpFilePath):
		os.remove(tmpFilePath)
	pass
	return ret
pass

################################################
#向资源文件写入一行内容，并且以"</resources>"结尾
###############################################
def writeIdsLine(srcFile,targetString):
	deleteLine(srcFile,"</resources>")
	deleteLine(srcFile,targetString)
	writeLine(srcFile,targetString)
	writeLine(srcFile,"</resources>")
pass

################################################
#创建资源文件，并且以写入
# "<?xml version="1.0" encoding="UTF-8"?>
# <resources>
# </resources>"内容
###############################################
def createIdsFile(filePath):
	if os.path.exists(filePath):
		pass
	else:
		f = open(filePath,'w')
		f.close()
		writeLine(filePath,'<?xml version="1.0" encoding="UTF-8"?>')
		writeLine(filePath,"<resources>")
		writeLine(filePath,"</resources>")
	pass
pass

################################################
#安全退出
###############################################
def safeQuit(signum, frame):
	print "###############################"
	print "Now safely Quit:%s"%glo.getValue('_reason')
	print "###############################"

	output = os.popen('find ./ -name "*.tiny" | xargs rm -rf')
	output = os.popen('rm -rf *.new')

	mailto_list = []
	mailto_list.append('xiangc@jumei.com')
	mailto_list.append('feiy1@jumei.com')
	title = glo.getValue("_title")
	if title:
		content = glo.getValue('_content','you branch need local lint check')
		content += (" where branch is :%s"%glo.getValue('branch'))
		send_mail(title,content,mailto_list)
	pass
	# output = os.popen('find ./ -name "*.new" | xargs rm -rf')
	# find ./ -name "*.tiny" | xargs rm -rf
	#find ./ -name "*.new" | xargs rm -rf
	#find ./ -name "*.orig" | xargs rm -rf
	exit()
pass

def send_mail(title,content, mailto_list):
	#############
	#要发给谁，这里发给1个人
	#####################
	#设置服务器，用户名、口令以及邮箱的后缀
	mail_host="mail.jumei.com"
	# mail_user="pythonm"
	# mail_pass="Wuhang0508."
	mail_postfix="jumei.com"
	mail_user="feiy1"
	mail_pass="113456Yf@@"
	######################
	'''''
    to_list:发给谁
    sub:主题
    content:内容
    send_mail("aaa@126.com","sub","content")
    '''
	if len(mailto_list) == 0:
		return False

	me=mail_user+"<"+mail_user+"@"+mail_postfix+">"
	msg = MIMEText(content,_charset='UTF-8')
	msg['Subject'] = title
	msg['From'] = me
	msg['To'] = ";".join(mailto_list)
	try:
		s = smtplib.SMTP()
		s.connect(mail_host)
		s.login(mail_user,mail_pass)
		s.sendmail(me, mailto_list, msg.as_string())
		s.close()
		return True
	except Exception, e:
		print str(e)
		return False

################################################
#主函数
###############################################
def main():
	safeQuit(None,None)
pass

if __name__ == '__main__':
	main()


#!/usr/bin/env python             
# coding: utf-8
################################################
# copy from :https://github.com/KeepSafe/android-resource-remover/blob/master/android_clean_app.py
###############################################

import argparse
import distutils.spawn
import os
import re
import subprocess
from lxml import etree
import lcba.globalVar as glo

import util

LAYOUT_IDS_PATH = None
JUST_CHECK = None

class Issue:
    """
    Stores a single issue reported by Android Lint
    """

    def __init__(self, filepath, remove_file):
        self.filepath = filepath
        self.remove_file = remove_file
        self.elements = []

    def __str__(self):
        return '{0} {1}'.format(self.filepath, self.elements)

    def __repr__(self):
        return '{0} {1}'.format(self.filepath, self.elements)

    def add_element(self, message):
        res_all = re.findall(self.pattern, message)
        if res_all:
            self._process_match(res_all)
        else:
            print("The pattern '%s' seems to find nothing in the error message '%s'. We can't find the resource and "
                  "can't remove it. The pattern might have changed, please check and report this in github issues." % (
                      self.pattern.pattern, message))


class UnusedResourceIssue(Issue):
    pattern = re.compile('The resource `?([^`]+)`? appears to be unused')

    def _process_match(self, match_result):
        # print "\n##########"
        bits = match_result[0].split('.')[-2:]
        self.elements.append((bits[0], bits[1]))
        # print match_result
        # print "##########"
        # print bits


class ExtraTranslationIssue(Issue):
    pattern = re.compile('The resource string \"`([^`]+)`\" has been marked as `translatable=\"false')

    def _process_match(self, match_result):
        self.elements.append(('string', match_result[0]))


def parse_args():
    """
    Parse command line arguments.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('--lint',
                        help='Path to the ADT lint tool. If not specified it assumes lint tool is in your path',
                        default='lint')
    parser.add_argument('--app',
                        help='Path to the Android app. If not specifies it assumes current directory is your Android '
                             'app directory',
                        default='.')
    parser.add_argument('--xml',
                        help='Path to the lint result. If not specifies linting will be done by the script',
                        default=None)
    parser.add_argument('--ignore-layouts',
                        help='Should ignore layouts',
                        action='store_true')
    args = parser.parse_args()
    print args.ignore_layouts
    return args.lint, args.app, args.xml, args.ignore_layouts


def run_lint_command():
    """
    Run lint command in the shell and save results to lint-result.xml
    """
    lint, app_dir, lint_result, ignore_layouts = parse_args()
    if not lint_result:
        if not distutils.spawn.find_executable(lint):
            raise Exception(
                '`%s` executable could not be found and path to lint result not specified. See --help' % lint)
        lint_result = os.path.join(app_dir, 'lint-result.xml')
        call_result = subprocess.call([lint, app_dir, '--xml', lint_result])
        if call_result > 0:
            print('Running the command failed with result %s. Try running it from the console.'
                  ' Arguments for subprocess.call: %s' % (call_result, [lint, app_dir, '--xml', lint_result]))
    else:
        if not os.path.isabs(lint_result):
            lint_result = os.path.join(app_dir, lint_result)
    lint_result = os.path.abspath(lint_result)
    return lint_result, app_dir, ignore_layouts


def get_manifest_string_refs(manifest_path):
    pattern = re.compile('="@string/([^"]+)"')
    with open(manifest_path, 'r') as f:
        data = f.read()
        refs = set(re.findall(pattern, data))
        return [x.replace('/', '.') for x in refs]


def _get_issues_from_location(issue_class, locations, message):
    issues = []
    # print "_get_issues_from_location()...begin:%s"%message
    for location in locations:
        filepath = location.get('file')
        # if the location contains line and/or column attribute not the entire resource is unused.
        # that's a guess ;)
        # TODO stop guessing
        remove_entire_file = (location.get('line') or location.get('column')) is None
        issue = issue_class(filepath, remove_entire_file)
        issue.add_element(message)
        # fixme 存在这个问题，这个drawable xml都没有用，但是出现了line，又引用了其他删除的文件
        # if issue.elements[0][0] == 'drawable' and 'drawable' in filepath:
        #     print "we should remove file:%s, what message is:%s"%(issue.filepath,message)
        #     issue.remove_file = True
        # pass
        if False == remove_entire_file and issue.elements[0][1] in filepath:
            print "we should remove file:%s, what message is:%s" % (issue.filepath, message)
            issue.remove_file = True
        pass
        issues.append(issue)

    # print "_get_issues_from_location()...size:%s....end:%s"%(len(issues),message)
    return issues


def parse_lint_result(lint_result_path, manifest_path):
    """
    Parse lint-result.xml and create Issue for every problem found except unused strings referenced in AndroidManifest
    """
    unused_string_pattern = re.compile('The resource `R\.string\.([^`]+)` appears to be unused')
    mainfest_string_refs = get_manifest_string_refs(manifest_path)
    root = etree.parse(lint_result_path).getroot()
    issues = []

    for issue_xml in root.findall('.//issue[@id="UnusedResources"]'):
        message = issue_xml.get('message')
        unused_string = re.match(unused_string_pattern, issue_xml.get('message'))
        has_string_in_manifest = unused_string and unused_string.group(1) in mainfest_string_refs
        if not has_string_in_manifest:
            # print issue_xml.findall('location')
            issues.extend(_get_issues_from_location(UnusedResourceIssue,
                                                    issue_xml.findall('location'),
                                                    message))
            # print "after extend:size=%s"%len(issues)

    for issue_xml in root.findall('.//issue[@id="ExtraTranslation"]'):
        message = issue_xml.get('message')
        if re.findall(ExtraTranslationIssue.pattern, message):
            issues.extend(_get_issues_from_location(ExtraTranslationIssue,
                                                    issue_xml.findall('location'),
                                                    message))

    return issues


def remove_resource_file(issue, filepath, ignore_layouts):
    """
    Delete a file from the filesystem
    """
    print "remove_resource_file()...%s --> %s %s" % (issue.elements[0][0], filepath, ignore_layouts)
    if os.path.exists(filepath) and (ignore_layouts is False or issue.elements[0][0] != 'layout'):
        if JUST_CHECK:
            title = "you have unUnused Resource from lint check!!!"
            content =  "please local run:python lcba/lintResource.py to delete unUsedResources!!!"
            print title
            print content
            glo.set_value('_reason',title)
            glo.set_value('_title',title)
            glo.set_value('_content',content)
            util.safeQuit(None,None)
        pass

        # tory fix 如果是资源文件保存id
        if issue.elements[0][0] == 'layout':
            if not LAYOUT_IDS_PATH:
                print "you need to set the id path to save the layout @+id/***"
                util.safeQuit(None,None)
            pass
            util.createIdsFile(LAYOUT_IDS_PATH)
            print "remove_resource_file()...delete layout:%s where idsPath:%s" % (filepath, ids_path)
            with open(filepath) as f:
                for line in f.readlines():
                    # print "layout_line:%s"%line
                    #多行并列的情况怎么搞
                    pattern = re.compile('@\+id/\\b[^"]+')
                    match_result=re.findall(pattern,line)
                    #print "match_resutl:%s"%match_result
                    if match_result:
                        bits = match_result[0].rsplit('/',2)
                        #print "bits0:%s bits1:%s"%(bits[0],bits[1])
                        line = '<item name="%s" type="id" />' %bits[1]
                        print "write:%s to file:%s" % (line, ids_path)
                        util.writeIdsLine(ids_path, line)
                    pass
                pass
            pass
        pass
        print('removing resource: {0}'.format(filepath))
        os.remove(os.path.abspath(filepath))


def remove_resource_value(issue, filepath, ignore_layouts_value):
    """
    Read an xml file and remove an element which is unused, then save the file back to the filesystem
    """
    # if os.path.exists(filepath):
    # tory ignore layouts 暂时不处理layout的抽取
    print "remove_resource_value()...%s --> %s" % (issue.elements[0][0], filepath)
    if os.path.exists(filepath) and (ignore_layouts_value is False or issue.elements[0][0] != 'layout'):
        for element in issue.elements:
            print('removing {0} from resource {1}'.format(element, filepath))
            parser = etree.XMLParser(remove_blank_text=False, remove_comments=False,
                                     remove_pis=False, strip_cdata=False, resolve_entities=False)
            tree = etree.parse(filepath, parser)
            root = tree.getroot()
            for unused_value in root.findall('.//{0}[@name="{1}"]'.format(element[0], element[1])):
                root.remove(unused_value)
            with open(filepath, 'wb') as resource:
                tree.write(resource, encoding='utf-8', xml_declaration=True)


def remove_unused_resources(issues, app_dir, ignore_layouts_file, ignore_layouts_value):
    """
    Remove the file or the value inside the file depending if the whole file is unused or not.
    """
    # print "remove_unused_resources()...size:%s dir:%s file:%s value:%s"%(len(issues),app_dir,ignore_layouts_file,
    #                                                         ignore_layouts_value)
    for index, issue in enumerate(issues):
        # print "remove_unused_resources()...index:%s path:%s"%(index,issue.filepath)
        # for issue in issues:
        # 过滤lint报告中非本模块的 issue
        if app_dir not in issue.filepath:
            print "ignore issue:%s" % issue.filepath
            continue
        pass

        filepath = os.path.join(app_dir, issue.filepath)
        # print "issue.filepath:%s  app_dir:%s filepath:%s"%(issue.filepath,app_dir,filepath)
        if issue.remove_file:
            remove_resource_file(issue, filepath, ignore_layouts_file)
        else:
            remove_resource_value(issue, filepath, ignore_layouts_value)


################################################
# 删除资源，包括文件和重复的字符
###############################################
def deleteRes(app_dir,lint_result_path=None, manifest_path=None,out_ids_path=None,just_check=False,
              ignore_layouts_file=False,
              ignore_layouts_value=True):
    # print "deleteRes()...lint_path:%s app_dir:%s file:%s value:%s"%(lint_result_path,app_dir,ignore_layouts_file,
    # ignore_layouts_value)
    if not app_dir:
        print "###########ERROR###############"
        print "deleteRes()...app_dir param error:%s"%app_dir
        return
    pass

    for dirpath, dirnames, filenames in os.walk(app_dir):
        #print "dirpath:%s dirnames:%s filenames:%s" %(dirpath,dirnames,filenames)
        for filename in filenames:
            if not manifest_path and "AndroidManifest.xml" in filename:
                if "build" not in dirpath and "intermediates" not in dirpath and "generated" not in dirpath and "outputs" not in dirpath and "gen" not in dirpath:
                    manifest_path = os.path.join(dirpath,filename)
                    print "find AndroidManifest.xml:%s"%manifest_path
                pass
            elif not lint_result_path and "lint-results" in filename:
                if "build" in dirpath and filename.endswith('.xml'):
                    lint_result_path = os.path.join(dirpath,filename)
                    print "find lint reprt:%s"%lint_result_path
                pass
        pass
        if not out_ids_path and 'res/values' in dirpath:
            if "build" not in dirpath and "intermediates" not in dirpath and "generated" not in dirpath and "outputs" not in dirpath and "gen" not in dirpath:
                out_ids_path = os.path.join(dirpath,"lint_ids.xml")
                print "find out_ids_pat:%s"%out_ids_path
            pass
        pass
    pass

    if not lint_result_path or not manifest_path or not out_ids_path:
        print "###########ERROR###############"
        print "deleteRes()...param check error lint_result_path:%s manifest_path:%s out_ids_path:%s"%(lint_result_path,
                                                                                                    manifest_path,out_ids_path)
        return
    pass

    global LAYOUT_IDS_PATH
    LAYOUT_IDS_PATH = out_ids_path
    global JUST_CHECK
    JUST_CHECK = just_check
    issues = parse_lint_result(lint_result_path, manifest_path)
    print "deleteRes()...size:%s" % len(issues)
    remove_unused_resources(issues, app_dir, ignore_layouts_file, ignore_layouts_value)
pass


##############
# python lint_cleanup.py --xml build/reports/lint-results.xml --app JuMeiYouPinV1.0.0/
###############
def main():
    curPath = '/Users/tory/working/Codes/JUMEI/android/app/'
    deleteRes(app_dir=curPath)
pass

if __name__ == '__main__':
    main()



#!/usr/bin/env python             
# coding: utf-8
#####使用字典来对全局跨文件的变量进行访问

_global_dict = {}

def set_value(key,value):
    """ 定义一个全局变量 """
    # getValue(key,value)
    _global_dict[key] = value


def get_value(key,defValue=None):
    """ 获得一个全局变量,不存在则返回默认值 """
    try:
        return _global_dict[key]
    except KeyError:
        return defValue

def getValue(key,defValue=None):
    ret = None
    try:
       ret = get_value(key,None)
    except NameError:
        pass
    if not ret:
        ret = get_value(key,defValue)
    pass
    return ret
pass




#!/usr/bin/env python
# coding: utf-8
import os
import sys
import util

################################################
#AndResGuard 使用7zip和zipalign来压缩资源
#fixme 通过identify 来定位资源的要添加到白名单
###############################################
def useAndResGuard(inputApk,outDir,configPath=None):
    commandLine = "java -jar "

    curPath = os.path.dirname(os.path.realpath(__file__))
    andResJarPath = os.path.join(curPath,'AndResGuard-cli-1.2.0.jar')
    if not os.path.exists(andResJarPath):
        print "AndResGuard-cli-1.2.0.jar not exists !!!"
        util.safeQuit(None, None)
    pass
    commandLine = commandLine + " %s "%andResJarPath

    if not os.path.exists(inputApk):
        print "inputApk:%s not exists !!!" %inputApk
        return
    pass
    commandLine = commandLine + (" %s "%inputApk)

    print "configPath:%s"%configPath
    if not configPath or not os.path.exists(configPath):
        configPath = os.path.join(curPath,'AndResGuard_config.xml')
    if not os.path.exists(configPath):
        print "AndResGuard_config.xml not exists!!! path:%s"%configPath
        util.safeQuit(None, None)
    pass
    commandLine = commandLine + " -config %s "%configPath

    commandLine = commandLine + " -out %s "%outDir

    output = os.popen('which 7za')
    zipTool =  output.read()
    if zipTool.strip() == '':
        print "you have not install 7za command!!!"
        print "window： 对于window应下载命名行版本，若将7za指定到环境变量，即无须设置。地址：http://sparanoid.com/lab/7z/download.html"
        print "linux：sudo apt-get install p7zip-full"
        print "mac：brew install p7zip"
        util.safeQuit(None, None)
    pass

    zipAlign_path = os.path.join(curPath,'zipalign')
    if not os.path.exists(zipAlign_path):
        print "not find zipalign tools,zipalign may in your $ANDROID_SDK/build-tools/25.0.1/zipalign"
        print "please config const.ZIPALIGN_PATH !!!"
        util.safeQuit(None, None)
    pass
    commandLine = commandLine + " -zipalign %s "%zipAlign_path

    # print commandLine
    # (status, output) = commands.getstatusoutput(commandLine)
    # print status,output
    print "###############################"
    print "Now begin sign and zipalign : %s"%commandLine
    print "###############################"
    os.system(commandLine)
# output = os.popen(commandLine)
# print output.read()
pass

def main():
    inputApk = '/Users/tory/working/Codes/JUMEI/android/app/build/outputs/apk/app-debug.apk'
    outDir = '/Users/tory/working/Codes/JUMEI/android/lcba/app'
    outDir = os.path.join(outDir,"JumeiOut")
    useAndResGuard(inputApk,outDir)
pass

if __name__ == '__main__':
    main()


#!/usr/bin/env python
from __future__ import print_function
from setuptools import setup, find_packages

setup(
    name="lcba",
    version="0.1.6",
    author="tory you",
    author_email="feiy1@jumei.com",
    description="lint clean android and compress android resources",
    long_description="""
    LCBA is a fineresource project use lint delete unUsed file and value. And this we will compress the picture for
    save the disk.
    """,
    # long_description=open('README.md').read(),
    license="MIT",
    url="https://www.baidu.com",
    packages=['lcba'],
    package_dir={'lcba': 'lcba'},
    # package_data={'lcba': ['html/*.html']},
    package_data={'lcba': ['*.html','*.jar','*.xml','zipalign'],
                  '': ['*.md'],},
    install_requires=[
        "lxml",
        # lxml_requirement
    ],
    keywords="lint resource",
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Topic :: Text Processing :: Indexing",
        "Topic :: Utilities",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
    ],
)